@@ -1,8 +1,9 @@
+import datetime as dt
 import json
-from datetime import datetime, timedelta
 from unittest.mock import Mock, patch
 
 import dhooks_lite
+from pytz import utc
 
 from django.core.cache import cache
 from django.db import models
@@ -22,35 +23,47 @@ from ..models import (
     Timer,
     _task_calc_staging_system,
 )
-from . import (
-    LoadTestDataMixin,
-    add_permission_to_user_by_name,
-    create_fake_staging_system,
-    create_fake_timer,
-    create_test_user,
+from .testdata.factory import (
+    create_notification_rule,
+    create_scheduled_notification,
+    create_staging_system,
+    create_timer,
+    create_user,
 )
+from .testdata.fixtures import LoadTestDataMixin
+from .utils import add_permission_to_user_by_name
 
 MODULE_PATH = "structuretimers.models"
 
 
 class TestTimer(LoadTestDataMixin, NoSocketsTestCase):
-    def test_str(self):
+    def test_str_1(self):
         timer = Timer(
             structure_name="Test",
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=datetime(2020, 8, 6, 13, 25),
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
         expected = 'Armor timer for Raitaru "Test" in Abune @ 2020-08-06 13:25'
         self.assertEqual(str(timer), expected)
 
+    def test_str_2(self):
+        timer = Timer(
+            structure_name="Test",
+            timer_type=Timer.Type.PRELIMINARY,
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_raitaru,
+        )
+        expected = 'Preliminary timer for Raitaru "Test" in Abune'
+        self.assertEqual(str(timer), expected)
+
     def test_structure_display_name_1(self):
         timer = Timer(
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=datetime(2020, 8, 6, 13, 25),
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
         expected = "Raitaru in Abune"
         self.assertEqual(timer.structure_display_name, expected)
@@ -61,7 +74,7 @@ class TestTimer(LoadTestDataMixin, NoSocketsTestCase):
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             location_details="P5-M3",
-            date=datetime(2020, 8, 6, 13, 25),
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
         expected = "Raitaru in Abune near P5-M3"
         self.assertEqual(timer.structure_display_name, expected)
@@ -72,7 +85,7 @@ class TestTimer(LoadTestDataMixin, NoSocketsTestCase):
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=datetime(2020, 8, 6, 13, 25),
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
         expected = 'Raitaru "Big Boy" in Abune'
         self.assertEqual(timer.structure_display_name, expected)
@@ -109,8 +122,8 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
 
     @patch(MODULE_PATH + "._task_schedule_notifications_for_timer")
     def test_schedule_notifications_for_new_timers(self, mock_schedule_notifications):
-        timer = create_fake_timer(
-            date=now() + timedelta(hours=4),
+        timer = create_timer(
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             enabled_notifications=True,
@@ -124,7 +137,7 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
         self, mock_schedule_notifications
     ):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
@@ -135,8 +148,8 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
         with patch(
             MODULE_PATH + "._task_schedule_notifications_for_timer"
         ) as mock_schedule_notifications:
-            timer = create_fake_timer(
-                date=now() + timedelta(hours=4),
+            timer = create_timer(
+                date=now() + dt.timedelta(hours=4),
                 eve_solar_system=self.system_abune,
                 structure_type=self.type_astrahus,
             )
@@ -144,7 +157,7 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
         with patch(
             MODULE_PATH + "._task_schedule_notifications_for_timer"
         ) as mock_schedule_notifications:
-            timer.date = now() + timedelta(hours=3)
+            timer.date = now() + dt.timedelta(hours=3)
             timer.save()
             self.assertTrue(mock_schedule_notifications.called)
             _, kwargs = mock_schedule_notifications.return_value.apply_async.call_args
@@ -154,8 +167,8 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
         with patch(
             MODULE_PATH + "._task_schedule_notifications_for_timer"
         ) as mock_schedule_notifications:
-            timer = create_fake_timer(
-                date=now() + timedelta(hours=4),
+            timer = create_timer(
+                date=now() + dt.timedelta(hours=4),
                 eve_solar_system=self.system_abune,
                 structure_type=self.type_astrahus,
             )
@@ -163,10 +176,39 @@ class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
         with patch(
             MODULE_PATH + "._task_schedule_notifications_for_timer"
         ) as mock_schedule_notifications:
-            timer.date = now() + timedelta(hours=3)
+            timer.date = now() + dt.timedelta(hours=3)
             timer.structure_name = "Some fancy name"
             self.assertFalse(mock_schedule_notifications.called)
 
+    @patch(MODULE_PATH + "._task_schedule_notifications_for_timer")
+    def test_dont_schedule_notifications_for_new_preliminary_timers(
+        self, mock_schedule_notifications
+    ):
+        # when
+        create_timer(timer_type=Timer.Type.PRELIMINARY)
+        # then
+        self.assertFalse(mock_schedule_notifications.called)
+
+    @patch(MODULE_PATH + "._task_schedule_notifications_for_timer")
+    def test_remove_scheduled_notifications_when_timer_changed_to_prelimary(
+        self, mock_schedule_notifications
+    ):
+        # given
+        rule = create_notification_rule(is_enabled=False)
+        timer = create_timer(date=now() + dt.timedelta(hours=4))
+        notification = create_scheduled_notification(
+            notification_rule=rule, timer=timer
+        )
+        mock_schedule_notifications.reset()
+        # when
+        timer.timer_type = Timer.Type.PRELIMINARY
+        timer.save()
+        # then
+        self.assertFalse(mock_schedule_notifications.called)
+        self.assertFalse(
+            ScheduledNotification.objects.filter(pk=notification.pk).exists()
+        )
+
 
 @patch(MODULE_PATH + "._task_schedule_notifications_for_timer", Mock)
 class TestTimerSaveXCalcDistances(LoadTestDataMixin, NoSocketsTestCase):
@@ -181,7 +223,7 @@ class TestTimerSaveXCalcDistances(LoadTestDataMixin, NoSocketsTestCase):
     def test_should_calc_distances_when_created(self, mock_calc_distances):
         # when
         timer = Timer.objects.create(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
@@ -194,8 +236,8 @@ class TestTimerSaveXCalcDistances(LoadTestDataMixin, NoSocketsTestCase):
     def test_should_recalc_distances_when_solar_system_has_changed(
         self, mock_calc_distances
     ):
-        timer = create_fake_timer(
-            date=now() + timedelta(hours=4),
+        timer = create_timer(
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
@@ -209,8 +251,8 @@ class TestTimerSaveXCalcDistances(LoadTestDataMixin, NoSocketsTestCase):
     def test_should_not_recalc_distances_when_solar_system_unchanged(
         self, mock_calc_distances
     ):
-        timer = create_fake_timer(
-            date=now() + timedelta(hours=4),
+        timer = create_timer(
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
@@ -226,9 +268,9 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
-        cls.user_1 = create_test_user(cls.character_1)
-        cls.user_2 = create_test_user(cls.character_2)
-        cls.user_3 = create_test_user(cls.character_3)
+        cls.user_1 = create_user(cls.character_1)
+        cls.user_2 = create_user(cls.character_2)
+        cls.user_3 = create_user(cls.character_3)
         cls.user_1 = add_permission_to_user_by_name(
             "structuretimers.create_timer", cls.user_1
         )
@@ -244,7 +286,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_creator_can_edit_own_timer(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             user=self.user_1,
@@ -253,7 +295,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_manager_can_edit_other_timers(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             user=self.user_1,
@@ -262,7 +304,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_non_manager_can_not_edit_other_timer(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             user=self.user_1,
@@ -272,7 +314,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
     """
     def test_user_with_basic_access_can_view_normal_timer(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             user=self.user_1,
@@ -281,7 +323,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_user_can_not_view_corp_restricted_timer_from_other_corp(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_corporation=self.corporation_1,
@@ -292,7 +334,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_user_can_view_corp_restricted_timer_from_same_corp(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_corporation=self.corporation_1,
@@ -303,7 +345,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_user_can_not_view_alliance_restricted_timer_from_other_alliance(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_alliance=self.alliance_1,
@@ -314,7 +356,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_opsec_user_can_view_opsec_timer(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             is_opsec=True,
@@ -324,7 +366,7 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 
     def test_non_opsec_user_can_not_view_opsec_timer(self):
         timer = Timer(
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             is_opsec=True,
@@ -338,17 +380,17 @@ class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
 @patch("structuretimers.managers.STRUCTURETIMERS_TIMERS_OBSOLETE_AFTER_DAYS", 1)
 class TestTimerManger(LoadTestDataMixin, NoSocketsTestCase):
     def test_delete_old_timer(self):
-        timer_1 = create_fake_timer(
+        timer_1 = create_timer(
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             date=now(),
         )
-        timer_2 = create_fake_timer(
+        timer_2 = create_timer(
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=now() - timedelta(days=1, seconds=1),
+            date=now() - dt.timedelta(days=1, seconds=1),
         )
         result = Timer.objects.delete_obsolete()
         self.assertEqual(result, 1)
@@ -446,9 +488,9 @@ class TestTimerSendNotification(LoadTestDataMixin, NoSocketsTestCase):
 class TestTimerQuerySet(LoadTestDataMixin, NoSocketsTestCase):
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
-        self.timer_1 = create_fake_timer(
+        self.timer_1 = create_timer(
             structure_name="Timer 1",
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
@@ -456,9 +498,9 @@ class TestTimerQuerySet(LoadTestDataMixin, NoSocketsTestCase):
             timer_type=Timer.Type.ARMOR,
             objective=Timer.Objective.FRIENDLY,
         )
-        self.timer_2 = create_fake_timer(
+        self.timer_2 = create_timer(
             structure_name="Timer 2",
-            date=now() - timedelta(hours=8),
+            date=now() - dt.timedelta(hours=8),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
@@ -710,7 +752,7 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, NoSocketsTestCase):
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
         self.webhook = DiscordWebhook.objects.create(name="Dummy", url="my-url")
-        self.timer = create_fake_timer(
+        self.timer = create_timer(
             structure_name="Test",
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
@@ -740,6 +782,11 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, NoSocketsTestCase):
         self.timer.timer_type = Timer.Type.ARMOR
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
+    def test_should_never_match_without_date(self):
+        # given
+        self.timer.date = None
+        self.assertFalse(self.rule.is_matching_timer(self.timer))
+
     def test_require_objectives(self):
         # do not process if it does not match
         self.rule.require_objectives = [Timer.Objective.HOSTILE]
@@ -882,9 +929,9 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, NoSocketsTestCase):
         when one rule conforms with timer
         then qs contains only conforming rule
         """
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Test Timer",
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
@@ -902,9 +949,9 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, NoSocketsTestCase):
         when no rule conforms with timer
         then qs is empty
         """
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Test Timer",
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
@@ -922,9 +969,9 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, NoSocketsTestCase):
         when one rule conforms with timer
         then qs contains only conforming rule
         """
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Test Timer",
-            date=now() + timedelta(hours=4),
+            date=now() + dt.timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
@@ -1003,8 +1050,8 @@ class TestNotificationRuleSave(LoadTestDataMixin, NoSocketsTestCase):
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
-        timer = create_fake_timer(
-            date=now() + timedelta(hours=4),
+        timer = create_timer(
+            date=now() + dt.timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
@@ -1012,7 +1059,7 @@ class TestNotificationRuleSave(LoadTestDataMixin, NoSocketsTestCase):
             timer=timer,
             notification_rule=rule,
             timer_date=timer.date,
-            notification_date=timer.date - timedelta(minutes=10),
+            notification_date=timer.date - dt.timedelta(minutes=10),
         )
         rule.trigger = NotificationRule.Trigger.NEW_TIMER_CREATED
         rule.scheduled_time = None
@@ -1047,16 +1094,16 @@ class TestNotificationRuleMultiselectDisplay(NoSocketsTestCase):
 @patch(MODULE_PATH + ".EveSolarSystem.distance_to", lambda *args, **kwargs: 4.257e16)
 @patch(MODULE_PATH + ".EveSolarSystem.jumps_to", lambda *args, **kwargs: 3)
 @patch(MODULE_PATH + "._task_calc_staging_system", wraps=_task_calc_staging_system)
-@override_settings(CELERY_ALWAYS_EAGER=True)
+@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
 class TestStagingSystem(LoadTestDataMixin, NoSocketsTestCase):
     def test_should_calc_distances(self, spy_task_calc_staging_system):
         # given
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Test",
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=datetime(2020, 8, 6, 13, 25),
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
         # when
         staging_system = StagingSystem.objects.create(
@@ -1073,16 +1120,14 @@ class TestStagingSystem(LoadTestDataMixin, NoSocketsTestCase):
         self, spy_task_calc_staging_system
     ):
         # given
-        create_fake_timer(
+        create_timer(
             structure_name="Test",
             timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            date=datetime(2020, 8, 6, 13, 25),
-        )
-        staging_system = create_fake_staging_system(
-            eve_solar_system=self.system_enaluri
+            date=dt.datetime(2020, 8, 6, 13, 25, tzinfo=utc),
         )
+        staging_system = create_staging_system(eve_solar_system=self.system_enaluri)
         # when
         staging_system.save()
         # then
