@@ -4,7 +4,7 @@ from copy import deepcopy
 from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
 from django.core.exceptions import PermissionDenied
-from django.shortcuts import HttpResponse, get_object_or_404
+from django.shortcuts import get_object_or_404
 from django.urls import reverse
 from django.utils.html import format_html
 from django.utils.safestring import mark_safe
@@ -54,12 +54,17 @@ class TimerListView(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
     def get_context_data(self, **kwargs):
         staging_systems_qs = StagingSystem.objects.select_related(
             "eve_solar_system", "eve_solar_system__eve_constellation__eve_region"
-        )
-        try:
-            selected_staging_system = staging_systems_qs.get(
-                eve_solar_system__name=self.request.GET.get("staging")
-            )
-        except (StagingSystem.DoesNotExist, ValueError):
+        ).filter(eve_solar_system__isnull=False)
+        selected_staging_system = None
+        staging_system_name = self.request.GET.get("staging")
+        if staging_system_name:
+            try:
+                selected_staging_system = staging_systems_qs.get(
+                    eve_solar_system__name=self.request.GET.get("staging")
+                )
+            except (StagingSystem.DoesNotExist, ValueError):
+                pass
+        if not selected_staging_system:
             selected_staging_system = staging_systems_qs.filter(is_main=True).first()
             if not selected_staging_system:
                 selected_staging_system = staging_systems_qs.first()
@@ -412,15 +417,10 @@ class CopyTimerView(CreateTimerView):
         return kwargs
 
 
-class RemoveTimerView(EditTimerMixin, TimerManagementView, DeleteView):
-    def __init__(self) -> None:
-        self.object = None
-
-    def delete(self, request, *args, **kwargs) -> HttpResponse:
-        self.object = self.get_object()
-        response = super().delete(request, *args, **kwargs)
-        self.send_success_message(_("Removed"))
-        return response
+class RemoveTimerView(
+    EditTimerMixin, LoginRequiredMixin, PermissionRequiredMixin, DeleteView
+):
+    model = Timer
 
     def get_success_url(self) -> str:
         return self.object.get_absolute_url()
